import PublicClient from '../clients/types/publicClient';
import PrivateClient from '../clients/types/privateClient';
import CustomClient from '../clients/types/customClient';

const logger = require('../logger');

class ChannelBase {
  id: string;
  clients: any[] = [];
  broadcastConditions: string = "true";
  channelContent: JSON = JSON.parse('{}');
  explicitRemoval: boolean = false;

  constructor(id: string) {
    this.id = id;
    logger.accessLog.info('Channel Created', {channelId: id});
  }

  broadcastMessage(from: PublicClient|PrivateClient|CustomClient, message: object) {
    for (const to of this.clients) {
      if (this.messageTransactionPossible(from, to)) {
        to.ws.send(JSON.stringify(message));
        logger.accessLog.info(`sent to ${to.id}`, { data: {message}});
      } else {
        logger.accessLog.info(`client is unable to send: ${to.id}`, { data: {message}});
      }
    }

    return {status: 'success', message: `message broadcast complete`};
  }

  messageTransactionPossible(from: PublicClient|PrivateClient|CustomClient, to: PublicClient|PrivateClient|CustomClient) {
    return eval(this.broadcastConditions);
  }

  addClient(client: PublicClient|PrivateClient|CustomClient) {
    if (this.clientExists(client.id)) {
      logger.errorLog.info('Client already exits in channel', {channelId: this.id, clientId: client.id});
      return {status: 'notice', message: 'client already exists in channel'};
    } else {
      this.clients.push(client);
      client.connectToChannel(this);
      logger.accessLog.info('Added client to channel', {channelId: this.id, clientId: client.id});
      return {status: 'success', message: 'client added'};
    }
  }

  clientExists(id: number) {
    for (const client of this.clients) {
      if (client.id === id) {
        return client;
      }
    }

    return null;
  }

  removeClient(id: number) {
    for (const client of this.clients) {
      if (client.id === id) {
        const index = this.clients.indexOf(client);
        this.clients.splice(index, 1);
        return true;
      }
    }

    return false;
  }
}

export default ChannelBase;
