const {winston, transports, createLogger, format} = require('winston');
const path = require('path');

// Set this to whatever, by default the path of the script.
const logPath = './logs/';

const tsFormat = () => (new Date().toISOString());
const logFormat = format.combine(format.timestamp(), format.json());

function loggerTransports(logLevel: string, logName = "") {
  let transport:any;
  if ("" === logName) {
    transport = new transports.Console({
      format: format.simple(),
      level: logLevel,
      timestamp: tsFormat
    });
  } else {
    transport = new transports.File({
      filename: path.join(logPath, `${logName}.log`),
      timestamp: tsFormat,
      level: logLevel
    });
  }

  return [ transport ];
}

const debugLog = createLogger({
  format: format.simple(),
  transports: loggerTransports('debug')
});

const errorLog = createLogger({
  format: logFormat,
  transports: loggerTransports('debug', 'errors')
});

const accessLog = createLogger({
  format: logFormat,
  transports: loggerTransports('info', 'access')
});

module.exports = {
  errorLog,
  accessLog,
  debugLog
};
